import json

from .errors import NoResultFound

from urllib.request import urlopen, Request
from urllib.error import HTTPError
from random import choice

__all__ = ["Waifu"]


class Waifu:
    """Represents the waifu class. Fetches image urls from `waifu.pics <https://waifu.pics/>`__."""

    base = "https://waifu.pics/api/sfw/"

    @classmethod
    def __image__(cls, url: str) -> str:

        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.3"
        }
        req = Request(url=url, headers=headers)

        try:
            page = urlopen(req)
        except HTTPError as e:
            raise NoResultFound(f"HTTP Error: {e.code}")

        res = json.loads(page.read())

        return res["url"]

    @classmethod
    def random(cls) -> str:
        """Returns waifus from random category"""

        select = choice(["waifu", "shinobu", "megumin", "neko"])
        return cls.__image__(cls.base + select)

    @classmethod
    def random_gif(cls) -> str:
        """Returns a random anime gif"""

        select = choice(
            [
                "bully",
                "cuddle",
                "cry",
                "hug",
                "awoo",
                "kiss",
                "lick",
                "pat",
                "smug",
                "bonk",
                "yeet",
                "blush",
                "smile",
                "wave",
                "highfive",
                "handhold",
                "nom",
                "bite",
                "glomp",
                "slap",
                "kill",
                "kick",
                "happy",
                "wink",
                "poke",
                "dance",
                "cringe",
            ]
        )
        return cls.__image__(cls.base + select)

    @classmethod
    def waifu(cls) -> str:
        """Returns untagged waifus"""

        return cls.__image__(cls.base + "waifu")

    @classmethod
    def shinobu(cls) -> str:
        """Returns waifus from shinobu category"""

        return cls.__image__(cls.base + "shinobu")

    @classmethod
    def megumin(cls) -> str:
        """Returns waifus from megumin category"""

        return cls.__image__(cls.base + "megumin")

    @classmethod
    def neko(cls) -> str:
        """Returns waifus from neko category"""

        return cls.__image__(cls.base + "neko")

    @classmethod
    def bully(cls) -> str:
        """Returns anime bully gifs"""

        return cls.__image__(cls.base + "bully")

    @classmethod
    def cuddle(cls) -> str:
        """Returns anime cuddle gifs"""

        return cls.__image__(cls.base + "cuddle")

    @classmethod
    def cry(cls) -> str:
        """Returns anime cry gifs"""

        return cls.__image__(cls.base + "cry")

    @classmethod
    def hug(cls) -> str:
        """Returns anime hug gifs"""

        return cls.__image__(cls.base + "hug")

    @classmethod
    def awoo(cls) -> str:
        """Returns anime awoo gifs"""

        return cls.__image__(cls.base + "awoo")

    @classmethod
    def kiss(cls) -> str:
        """Returns anime kiss gifs"""

        return cls.__image__(cls.base + "kiss")

    @classmethod
    def lick(cls) -> str:
        """Returns anime lick gifs"""

        return cls.__image__(cls.base + "lick")

    @classmethod
    def pat(cls) -> str:
        """Returns anime pat gifs"""

        return cls.__image__(cls.base + "pat")

    @classmethod
    def smug(cls) -> str:
        """Returns anime smug gifs"""

        return cls.__image__(cls.base + "smug")

    @classmethod
    def bonk(cls) -> str:
        """Returns anime bonk gifs"""

        return cls.__image__(cls.base + "bonk")

    @classmethod
    def yeet(cls) -> str:
        """Returns anime yeet gifs"""

        return cls.__image__(cls.base + "yeet")

    @classmethod
    def blush(cls) -> str:
        """Returns anime blush gifs"""

        return cls.__image__(cls.base + "blush")

    @classmethod
    def smile(cls) -> str:
        """Returns anime wave gifs"""

        return cls.__image__(cls.base + "wave")

    @classmethod
    def highfive(cls) -> str:
        """Returns anime highfive gifs"""

        return cls.__image__(cls.base + "highfive")

    @classmethod
    def handhold(cls) -> str:
        """Returns anime handhold gifs"""

        return cls.__image__(cls.base + "handhold")

    @classmethod
    def nom(cls) -> str:
        """Returns anime nom gifs"""

        return cls.__image__(cls.base + "nom")

    @classmethod
    def bite(cls) -> str:
        """Returns anime bite gifs"""

        return cls.__image__(cls.base + "bite")

    @classmethod
    def glomp(cls) -> str:
        """Returns anime glomp gifs"""

        return cls.__image__(cls.base + "glomp")

    @classmethod
    def slap(cls) -> str:
        """Returns anime slap gifs"""

        return cls.__image__(cls.base + "slap")

    @classmethod
    def kill(cls) -> str:
        """Returns anime kill gifs"""

        return cls.__image__(cls.base + "kill")

    @classmethod
    def kick(cls) -> str:
        """Returns anime kick gifs"""

        return cls.__image__(cls.base + "kick")

    @classmethod
    def happy(cls) -> str:
        """Returns anime happy gifs"""

        return cls.__image__(cls.base + "happy")

    @classmethod
    def wink(cls) -> str:
        """Returns anime wink gifs"""

        return cls.__image__(cls.base + "wink")

    @classmethod
    def poke(cls) -> str:
        """Returns anime pole gifs"""

        return cls.__image__(cls.base + "poke")

    @classmethod
    def dance(cls) -> str:
        """Returns anime dance gifs"""

        return cls.__image__(cls.base + "dance")

    @classmethod
    def cringe(cls) -> str:
        """Returns anime cringe gifs"""

        return cls.__image__(cls.base + "cringe")



# -*- coding: utf-8 -*-


class NoResultFound(Exception):
    """Raised if no result is found or there is a HTTP error"""

    pass


class TooManyRequests(Exception):
    """Raised when number of requests exceed than the expected"""

    pass



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#

import os
import sys

sys.path.insert(0, os.path.abspath("../../"))

# -- Project information -----------------------------------------------------

project = "animec"
copyright = "2021, Drift Asimov"
author = "Drift Asimov"

# The full version, including alpha/beta/rc tags
setup_file = (
    open("../../setup.py").read().split("version")[1].split('"')[1].split('"')[0]
)
release = str(setup_file)

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
    "sphinx.ext.extlinks",
]

autodoc_member_order = "bysource"

intersphinx_mapping = {
    "py": ("https://docs.python.org/3", None),
}

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# The master toctree document.
master_doc = "index"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "furo"

html_context = {
    "discord_invite": "https://discord.gg/x3qAZV3",
    "issues": "https://github.com/DriftAsimov/animec/issues",
}

html_theme_options = {
    "sidebar_hide_name": True,
    "navigation_with_keys": True
}

html_favicon = "../assets/animec.png"
html_logo = "../assets/animec.png"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".



from setuptools import setup, find_packages

version = "0.4.2"

with open("requirements.txt", "r") as f:
    requirements = f.read().split("\n")

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "Operating System :: OS Independent",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3",
    "Topic :: Education :: Testing",
    "Topic :: Software Development :: Libraries",
]

setup(
    name="animec",
    version=version,
    description="A module to get data about anime characters, news, info, lyrics and more.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DriftAsimov/animec",
    project_urls={
        "Documentation": "https://animec.readthedocs.io/en/latest/",
        "Issue tracker": "https://github.com/DriftAsimov/animec/issues",
        "Github": "https://github.com/DriftAsimov/animec",
    },
    author="DriftAsimov",
    author_email="driftasimov@gmail.com",
    license="MIT",
    classifiers=classifiers,
    keywords=[
        "animecharacter anime character myanimelist news animenews animeinfo music animelyrics lyrics"
    ],
    packages=find_packages(),
    install_requires=requirements,
)
